@@ -7,7 +7,7 @@ use diagnostics;
 require Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT = qw( auction );
-our $VERSION = '0.70';
+our $VERSION = '0.80';
 
 #Variables global to the package	
 my $maximize_total_benefit;
@@ -17,9 +17,9 @@ my $max_matrix_value;
 my $need_transpose;
 my ( $array1_size, $array2_size, $min_size, $max_size, $original_max_size );
 my ( $inicial_price, $iter_count_global, $iter_count_local, $epsilon_scaling );
-my ( $target, $output, $locked_list );
+my ( $target, $output );
 my ( %index_correlation, %assignned_object, %assignned_person, %price_object );
-my ( %objects_desired_by_this,  );
+my ( %objects_desired_by_this, %locked_list, %seen_person );
 
 sub auction { #						=> default values
 	my %args = ( matrix_ref				=> undef,     # reference to array: matrix N x M			                                     
@@ -34,12 +34,13 @@ sub auction { #						=> default values
 	$iter_count_global = 0;
 	$epsilon_scaling = 0;
 	$need_transpose = 0;
-	$locked_list = 0;
 	%index_correlation = ();
 	%assignned_object = ();
 	%assignned_person = ();
 	%price_object = ();
 	%objects_desired_by_this = ();
+	%locked_list = ();
+	%seen_person = ();
 	
 	my @matrix_input = @{$args{matrix_ref}};          # Input: Reference to the input matrix (NxM) = $min_size x $max_size
    
@@ -50,15 +51,15 @@ sub auction { #						=> default values
 	$max_size = $array1_size < $array2_size ? $array2_size : $array1_size ;
 	$original_max_size = $max_size;
 
-	$target = 'auction-' . $min_size . 'x' . $max_size . '-output.txt' ;
+	$target = 'auction-' . $array1_size . 'x' . $array2_size . '-output.txt' ;
 	
 	if ( $args{verbose} >= 8 ){ # print the verbose messages to $output file.
-		print "\n verbose = $args{verbose} ---> ";
+		print "\n verbose = $args{verbose} ---> print the verbose messages to <$target> file \n";
 		if ( open ( $output, '>', $target ) ) {
-			print " *** Open <$target> for writing. *** \n";
+			print "\n *** Open <$target> for writing. *** \n";
 		} else {
 			$args{verbose} = 7;
-			warn " *** Could not open <$target> for writing: $! *** \n";			
+			warn "\n *** Could not open <$target> for writing: $! *** \n";			
 		}
 	}	
    
@@ -114,6 +115,7 @@ sub auction { #						=> default values
    
 		%assignned_object = ();
 		%assignned_person = ();
+		%seen_person = ();
 		$epsilon_scaling++;
 		$iter_count_local = 0;
 
@@ -271,7 +273,7 @@ sub delete_multiple_columns { # if the column elements do not change the final r
 		}
 		print "\n";
 	}
-   
+
 	$max_size = $max_size - $number_of_columns_deleted;
 }
 
@@ -516,7 +518,7 @@ sub auctionRound {
 				}
 			}
 			
-			if ( not @updated_price and not $locked_list ) # if all prices are outdated
+			if ( not @updated_price and not $locked_list{$person} ) # if all prices are outdated
 			{				
 				if ( $epsilon_scaling <= 2 or $epsilon > 64 ) # 1000/(4**2) ~ 64 ; 1000000/(4**7) ~ 64 ; $max_matrix_value/(4**x) ~ 4**3 --> $max_matrix_value/(4**3) ~ (4**x) --> x ~ [ln ($max_matrix_value/(4**3))]/[ln(4)]
 				{
@@ -662,52 +664,50 @@ sub auctionRound {
 			
 			# Stores the bidding info for future use
 			$info{$Opt01ObjForPersonI}{$bid01ForPersonI} = $person if ( not defined $info{$Opt01ObjForPersonI}{$bid01ForPersonI} ); # get only one person with this bid			
+	
+			$bid02ForPersonI = $Opt02ValForPersonI - $Opt03ValForPersonI + $epsilon; # calculate $bid02ForPersonI to try to predict the values of the next round
+			$bid02ForPersonI = 2 * $epsilon if ( $Opt01ValForPersonI > ($Opt02ValForPersonI + $epsilon) or $Opt02ValForPersonI > ($Opt03ValForPersonI + $epsilon) );
 
-			if ( $max_size >= 6 )
-			{	
-				$bid02ForPersonI = $Opt02ValForPersonI - $Opt03ValForPersonI + $epsilon; # calculate $bid02ForPersonI to try to predict the values of the next round
-				$bid02ForPersonI = 2 * $epsilon if ( $Opt01ValForPersonI > ($Opt02ValForPersonI + $epsilon) or $Opt02ValForPersonI > ($Opt03ValForPersonI + $epsilon) );
+			$info{$Opt02ObjForPersonI}{$bid02ForPersonI} = $person if ( ++$seen_bid{$Opt01ObjForPersonI}{$bid01ForPersonI} >= 2 );
 
-				$info{$Opt02ObjForPersonI}{$bid02ForPersonI} = $person if ( ++$seen_bid{$Opt01ObjForPersonI}{$bid01ForPersonI} >= 2 );
-
-				if ( $epsilon_scaling >= 6 and $epsilon_scaling % 3 == 0 ) # filter the old list
-				{					
-					for my $object ( @objects_with_greater_benefits ){
-						next if ( $current_value{$object} >= $Opt03ValForPersonI );
-						next unless ( ($Opt03ValForPersonI - $current_value{$object}) > 2 * $min_size * $epsilon );
-						delete $objects_desired_by_this{$person}{$object}; # delete objects of little benefit from the old list
-						printf $output "<> PersonI = %3s ; *** delete the object %3s from the old list *** \n", $person, $object if ( $verbose >= 8 );
-					}
-					$locked_list++;
+			if ( $epsilon_scaling >= 6 and $epsilon_scaling % 2 == 0 and not $seen_person{$person}++ ) # filter the old list
+			{					
+				for my $object ( @objects_with_greater_benefits ){ # frequently check the quality of objects in the old list
+					next if ( $current_value{$object} >= $Opt03ValForPersonI );
+					next unless ( ($Opt03ValForPersonI - $current_value{$object}) > 2 * $min_size * $epsilon ); # critical value ($min_size * $epsilon) and safety margin (2).
+					delete $objects_desired_by_this{$person}{$object}; # delete objects of little benefit from the old list
+					printf $output "<> PersonI = %3s ; *** delete the object %3s from the old list *** \n", $person, $object if ( $verbose >= 8 );
+				}
+			}
+					
+			if ( not @updated_price ) # if all prices are outdated
+			{			
+				for my $object ( @objects_with_greater_benefits ){
+					next unless ( defined $objects_desired_by_this{$person}{$object} );
+					$objects_desired_by_this{$person}{$object} = $current_value{$object}; # update current values for all objects in the old list	
 				}
 					
-				if ( not @updated_price ) # if all prices are outdated
-				{			
-					for my $object ( @objects_with_greater_benefits ){
-						next unless ( defined $objects_desired_by_this{$person}{$object} );
-						$objects_desired_by_this{$person}{$object} = $current_value{$object}; # update current values for all objects in the old list	
-					}
-					
-					if ( $locked_list and defined $Opt01ObjForPersonI_new_list and $Opt03ValForPersonI == $Opt01ValForPersonI_new_list and defined $objects_desired_by_this{$person}{$Opt03ObjForPersonI} )
-					{
-						$objects_desired_by_this{$person}{$Opt01ObjForPersonI_new_list} = $current_value{$Opt01ObjForPersonI_new_list}; # add object to list
-						printf $output "<> PersonI = %3s ; *** add object %3s to the old list *** \n", $person, $Opt01ObjForPersonI_new_list if ( $verbose >= 8 );
-					}										
-					
-					$objects_desired_by_this{$person}{$Opt01ObjForPersonI} = $current_value{$Opt01ObjForPersonI}; # information about the most desired objects
-					$objects_desired_by_this{$person}{$Opt02ObjForPersonI} = $current_value{$Opt02ObjForPersonI} if (defined $Opt02ObjForPersonI);
-					$objects_desired_by_this{$person}{$Opt03ObjForPersonI} = $current_value{$Opt03ObjForPersonI} if (defined $Opt03ObjForPersonI);					
+				if ( defined $Opt01ObjForPersonI_new_list and $Opt03ValForPersonI == $Opt01ValForPersonI_new_list and defined $objects_desired_by_this{$person}{$Opt03ObjForPersonI} )
+				{
+					$objects_desired_by_this{$person}{$Opt01ObjForPersonI_new_list} = $current_value{$Opt01ObjForPersonI_new_list}; # add object to list
+					printf $output "<> PersonI = %3s ; *** add object %3s to the old list *** \n", $person, $Opt01ObjForPersonI_new_list if ( $verbose >= 8 );
 				}
+
+				$locked_list{$person}++ if ( $epsilon_scaling >= 6 ); # Lock the old list. 6 is the minimum $epsilon_scaling enough to find a possible solution?			
+					
+				$objects_desired_by_this{$person}{$Opt01ObjForPersonI} = $current_value{$Opt01ObjForPersonI}; # information about the most desired objects
+				$objects_desired_by_this{$person}{$Opt02ObjForPersonI} = $current_value{$Opt02ObjForPersonI} if (defined $Opt02ObjForPersonI);
+				$objects_desired_by_this{$person}{$Opt03ObjForPersonI} = $current_value{$Opt03ObjForPersonI} if (defined $Opt03ObjForPersonI);					
 			}
 			
 			if ( $verbose >= 8 ){
 				my @old_list = sort { $a <=> $b } @objects_with_greater_benefits;
 				my @new_list = sort { $a <=> $b } keys %{$objects_desired_by_this{$person}};
 				my @best_3_objects = ( $Opt01ObjForPersonI, $Opt02ObjForPersonI, $Opt03ObjForPersonI );
-				my $msg = $locked_list ? 'locked list' : '';
+				my $msg = $locked_list{$person} ? '[locked list] ' : '';
 
 				printf $output "<> PersonI = %3s ; %3s objects desired by this person (old list) = (@old_list) ; objects whose current values are still updated = (@updated_price) : %2s >= 1 ? \n", $person, scalar @old_list, scalar @updated_price;
-				printf $output "<> PersonI = %3s ; %3s objects desired by this person (new list) = (@new_list) ; \@best_3_objects = (@best_3_objects) $msg \n", $person, scalar @new_list if ( defined $Opt03ObjForPersonI );
+				printf $output "<> PersonI = %3s ; %3s objects desired by this person (new list) = (@new_list) $msg; \@best_3_objects = (@best_3_objects) \n", $person, scalar @new_list if ( defined $Opt03ObjForPersonI );
 				printf $output "<> PersonI = %3s chose ObjectJ = %3s ; \$bid01ForPersonI %10.5f = \$Opt01ValForPersonI %.5f - \$Opt02ValForPersonI %.5f + \$epsilon %.5f \n", $person, $Opt01ObjForPersonI, $bid01ForPersonI, $Opt01ValForPersonI, $Opt02ValForPersonI, $epsilon;
 				printf $output "<> PersonI = %3s chose ObjectJ = %3s ; \$bid02ForPersonI %10.5f = \$Opt02ValForPersonI %.5f - \$Opt03ValForPersonI %.5f + \$epsilon %.5f \n", $person, $Opt02ObjForPersonI, $bid02ForPersonI, $Opt02ValForPersonI, $Opt03ValForPersonI, $epsilon if ( defined $bid02ForPersonI );
 			}
@@ -1013,7 +1013,7 @@ __END__
 =head1 AUTHOR
 
     Claudio Fernandes de Souza Rodrigues
-	April 15, 2018
+	April 16, 2018
 	Sao Paulo, Brasil
 	claudiofsr@yahoo.com
 	
